package com.fsck.k9.provider

import app.k9mail.feature.widget.unread.BaseUnreadWidgetProvider

/**
 * IMPORTANT: The fully qualified name for this class must be
 *            `com.fsck.k9.provider.UnreadWidgetProvider`.
 *             Otherwise widgets created with older versions of the app using a different name
 *             will stop working or disappear.
 */
class UnreadWidgetProvider : BaseUnreadWidgetProvider()
