package app.k9mail.core.ui.compose.theme2.default

import androidx.compose.ui.unit.dp
import app.k9mail.core.ui.compose.theme2.ThemeElevations

/**
 * Default values for Material elevation taken from https://m3.material.io/styles/elevation/tokens
 */
val defaultThemeElevations = ThemeElevations(
    level0 = 0.dp,
    level1 = 1.dp,
    level2 = 3.dp,
    level3 = 6.dp,
    level4 = 8.dp,
    level5 = 12.dp,
)
