package net.thunderbird.android.widget.provider

import app.k9mail.feature.widget.message.list.BaseMessageListWidgetProvider

/**
 * IMPORTANT: The fully qualified name for this class must be
 *            `net.thunderbird.android.widget.provider.MessageListWidgetProvider`.
 *            Otherwise widgets created with older versions of the app using a different name
 *            will stop working or disappear.
 */
class MessageListWidgetProvider : BaseMessageListWidgetProvider()
